@@ -95,6 +95,7 @@ sub _construct {
 				   conflicts       => {},
 				   perl            => $perl,
 				   mb_version      => Module::Build->VERSION,
+				   build_elements  => [qw( PL support pm xs pod script )],
 				   install_types   => [qw( lib arch script bindoc libdoc )],
 				   installdirs     => 'site',
 				   include_dirs    => [],
@@ -252,6 +253,7 @@ sub notes {
        config_dir
        blib
        build_script
+       build_elements
        install_types
        install_sets
        install_path
@@ -354,48 +356,22 @@ sub dist_version {
   return $p->{dist_version} = $self->version_from_file($version_from);
 }
 
-sub dist_author {
-  my $self = shift;
-  my $p = $self->{properties};
-  return $p->{dist_author} if exists $p->{dist_author};
-  
-  # Figure it out from 'dist_version_from'
-  return unless $p->{dist_version_from};
-  my $fh = IO::File->new($p->{dist_version_from}) or return;
-  
-  my @author;
-  local $_;
-  while (<$fh>) {
-    next unless /^=head1\s+AUTHOR/ ... /^=/;
-    next if /^=/;
-    push @author, $_;
-  }
-  return unless @author;
-  
-  my $author = join '', @author;
-  $author =~ s/^\s+|\s+$//g;
-  return $p->{dist_author} = $author;
-}
+sub dist_author   { shift->_pod_parse('author')   }
+sub dist_abstract { shift->_pod_parse('abstract') }
 
-sub dist_abstract {
-  my $self = shift;
+sub _pod_parse {
+  my ($self, $part) = @_;
   my $p = $self->{properties};
-  return $p->{dist_abstract} if exists $p->{dist_abstract};
+  my $member = "dist_$part";
+  return $p->{$member} if exists $p->{$member};
   
-  # Figure it out from 'dist_version_from'
   return unless $p->{dist_version_from};
   my $fh = IO::File->new($p->{dist_version_from}) or return;
   
-  (my $package = $self->dist_name) =~ s/-/::/g;
-  
-  my $result;
-  local $_;
-  while (<$fh>) {
-    next unless /^=(?!cut)/ .. /^cut/;  # in POD
-    last if ($result) = /^(?:$package\s-\s)(.*)/;
-  }
-  
-  return $p->{dist_abstract} = $result;
+  require Module::Build::PodParser;
+  my $parser = Module::Build::PodParser->new(fh => $fh);
+  my $method = "get_$part";
+  return $p->{$member} = $parser->$method();
 }
 
 sub find_module_by_name {
@@ -555,7 +531,7 @@ sub write_config {
   $self->_write_dumper('prereqs', { map { $_, $self->$_() } @items });
   $self->_write_dumper('build_params', [$self->{args}, $self->{config}, $self->{properties}]);
 
-  $self->_persistent_hash_write('cleanup');
+  $self->_persistent_hash_write($_) foreach qw(notes cleanup);
 }
 
 sub requires       { shift()->{properties}{requires} }
@@ -577,6 +553,12 @@ sub prereq_failures {
 	next if !$status->{ok};
 	$status->{conflicts} = delete $status->{need};
 	$status->{message} = "Installed version '$status->{have}' of $modname conflicts with this distribution";
+
+      } elsif ($type eq 'recommends') {
+	next if $status->{ok};
+	$status->{message} = ($status->{have}
+			      ? "Version $status->{have} is installed, but we prefer to have $spec"
+			      : "Optional prerequisite $modname isn't installed");
       } else {
 	next if $status->{ok};
       }
@@ -596,9 +578,9 @@ sub check_prereq {
   
   foreach my $type (qw(requires build_requires conflicts recommends)) {
     next unless $failures->{$type};
-    my $prefix = $type eq 'recommends' ? 'WARNING' : 'ERROR';
+    my $prefix = $type eq 'recommends' ? '' : 'ERROR: ';
     while (my ($module, $status) = each %{$failures->{$type}}) {
-      warn "$prefix: $module: $status->{message}\n";
+      warn " * $prefix$status->{message}\n";
     }
   }
   
@@ -651,7 +633,7 @@ sub check_installed_status {
     
     $status{have} = $self->version_from_file($file);
     if ($spec and !$status{have}) {
-      @status{ qw(have message) } = (undef, "Couldn't find a \$VERSION in prerequisite '$file'");
+      @status{ qw(have message) } = (undef, "Couldn't find a \$VERSION in prerequisite $modname");
       return \%status;
     }
   }
@@ -668,7 +650,7 @@ sub check_installed_status {
     next if $op eq '>=' and !$version;  # Module doesn't have to actually define a $VERSION
     
     unless ($self->compare_versions( $status{have}, $op, $version )) {
-      $status{message} = "Version $status{have} is installed, but we need version $op $version";
+      $status{message} = "Version $status{have} of $modname is installed, but we need version $op $version";
       return \%status;
     }
   }
@@ -719,6 +701,8 @@ sub make_executable {
   }
 }
 
+sub _startperl { shift()->{config}{startperl} }
+
 sub print_build_script {
   my ($self, $fh) = @_;
   
@@ -733,9 +717,12 @@ sub print_build_script {
   }
 
   my $quoted_INC = join ",\n", map "     '$_'", @myINC;
+  my $shebang = $self->_startperl;
 
   print $fh <<EOF;
-$self->{config}{startperl}
+$shebang
+
+use strict;
 
 BEGIN {
   \$^W = 1;  # Use warnings
@@ -749,8 +736,12 @@ $quoted_INC
 
 use $build_package;
 
+if (-e 'Build.PL' and not $build_package->up_to_date("Build.PL", \$0)) {
+   warn "Warning: Build.PL has been altered.  You may need to run 'perl Build.PL' again.\n";
+}
+
 # This should have just enough arguments to be able to bootstrap the rest.
-my \$build = resume $build_package (
+my \$build = $build_package->resume (
   properties => {
     config_dir => '$q{config_dir}',
   },
@@ -782,6 +773,8 @@ sub create_build_script {
 }
 
 sub check_manifest {
+  return unless -e 'MANIFEST';
+  
   # Stolen nearly verbatim from MakeMaker.  But ExtUtils::Manifest
   # could easily be re-written into a modern Perl dialect.
 
@@ -821,7 +814,7 @@ sub _call_action {
   return if $self->{_completed_actions}{$action}++;
   local $self->{action} = $action;
   my $method = "ACTION_$action";
-  die "No action '$action' defined" unless $self->can($method);
+  die "No action '$action' defined, try running the 'help' action.\n" unless $self->can($method);
   return $self->$method();
 }
 
@@ -864,7 +857,7 @@ sub cull_args {
     $args{$_} = [ $args{$_} ] unless ref $args{$_};
     foreach my $arg ( @{$args{$_}} ) {
       $arg =~ /(\w+)=(.+)/
-	or die "Malformed '$_' argument: '$arg'";
+	or die "Malformed '$_' argument: '$arg' should be something like 'foo=bar'";
       $hash{$1} = $2;
     }
     $args{$_} = \%hash;
@@ -993,10 +986,11 @@ sub ACTION_test {
   $self->depends_on('code');
   
   # Do everything in our power to work with all versions of Test::Harness
-  local ($Test::Harness::switches,
-	 $Test::Harness::Switches,
-         $ENV{HARNESS_PERL_SWITCHES}) = ($p->{debugger} ? '-w -d' : '') x 3;
-
+  my $harness_switches = $p->{debugger} ? ' -w -d' : '';
+  local $Test::Harness::switches    = ($Test::Harness::switches    || '') . $harness_switches;
+  local $Test::Harness::Switches    = ($Test::Harness::Switches    || '') . $harness_switches;
+  local $ENV{HARNESS_PERL_SWITCHES} = ($ENV{HARNESS_PERL_SWITCHES} || '') . $harness_switches;
+  
   local ($Test::Harness::verbose,
 	 $Test::Harness::Verbose,
 	 $ENV{TEST_VERBOSE},
@@ -1065,18 +1059,12 @@ sub ACTION_code {
   $self->add_to_cleanup($blib);
   File::Path::mkpath( File::Spec->catdir($blib, 'arch') );
   
-  if ($self->{properties}{autosplit}) {
-    $self->autosplit_file($self->{properties}{autosplit}, $blib);
+  $self->autosplit_file($self->autosplit, $blib) if $self->autosplit;
+  
+  foreach my $element (@{$self->build_elements}) {
+    my $method = "process_${element}_files";
+    $self->$method();
   }
-  
-  $self->process_PL_files;
-  
-  $self->compile_support_files;
-  
-  $self->process_pm_files;
-  $self->process_xs_files;
-  $self->process_pod_files;
-  $self->process_script_files;
 }
 
 sub ACTION_build {
@@ -1085,7 +1073,7 @@ sub ACTION_build {
   $self->depends_on('docs');
 }
 
-sub compile_support_files {
+sub process_support_files {
   my $self = shift;
   my $p = $self->{properties};
   return unless $p->{c_source};
@@ -1104,7 +1092,7 @@ sub process_PL_files {
   
   while (my ($file, $to) = each %$files) {
     unless ($self->up_to_date( $file, $to )) {
-      $self->run_perl_script($file);
+      $self->run_perl_script($file, [], [@$to]);
       $self->add_to_cleanup(@$to);
     }
   }
@@ -1159,7 +1147,7 @@ sub find_PL_files {
     # 'PL_files' is given as a Unix file spec, so we localize_file_path().
     
     if (UNIVERSAL::isa($files, 'ARRAY')) {
-      return { map {$_, /^(.*)\.PL$/}
+      return { map {$_, [/^(.*)\.PL$/]}
 	       map $self->localize_file_path($_),
 	       @$files };
 
@@ -1177,12 +1165,12 @@ sub find_PL_files {
   }
   
   return unless -d 'lib';
-  return { map {$_, /^(.*)\.PL$/} @{ $self->rscan_dir('lib', qr{\.PL$}) } };
+  return { map {$_, [/^(.*)\.PL$/]} @{ $self->rscan_dir('lib', qr{\.PL$}) } };
 }
 
-sub find_pm_files { shift->_find_file_by_type('pm') }
-sub find_pod_files { shift->_find_file_by_type('pod') }
-sub find_xs_files { shift->_find_file_by_type('xs') }
+sub find_pm_files  { shift->_find_file_by_type('pm',  'lib') }
+sub find_pod_files { shift->_find_file_by_type('pod', 'lib') }
+sub find_xs_files  { shift->_find_file_by_type('xs',  'lib') }
 
 sub find_script_files {
   my $self = shift;
@@ -1199,18 +1187,22 @@ sub find_script_files {
 }
 
 sub _find_file_by_type {
-  my ($self, $type) = @_;
+  my ($self, $type, $dir) = @_;
+  
   if (my $files = $self->{properties}{"${type}_files"}) {
     # Always given as a Unix file spec
     return { map $self->localize_file_path($_), %$files };
   }
   
-  return unless -d 'lib';
-  return { map {$_, $_} @{ $self->rscan_dir('lib', qr{\.$type$}) } };
+  return {} unless -d $dir;
+  return { map {$_, $_}
+	   map $self->localize_file_path($_),
+	   @{ $self->rscan_dir($dir, qr{\.$type$}) } };
 }
 
 sub localize_file_path {
   my ($self, $path) = @_;
+  return $path unless $path =~ m{/};
   return File::Spec->catfile( split qr{/}, $path );
 }
 
@@ -1299,8 +1291,8 @@ sub manify_lib_pods {
   my $mandir = File::Spec->catdir( $self->blib, 'libdoc' );
   File::Path::mkpath( $mandir, 0, 0777 );
 
-  foreach my $file (keys %$files) {
-    my $manpage = $self->man3page_name( $file ) . '.' . $self->{config}{man3ext};
+  while (my ($file, $relfile) = each %$files) {
+    my $manpage = $self->man3page_name( $relfile ) . '.' . $self->{config}{man3ext};
     my $outfile = File::Spec->catfile( $mandir, $manpage);
     next if $self->up_to_date( $file, $outfile );
     print "Manifying $file -> $outfile\n";
@@ -1315,7 +1307,7 @@ sub _find_pods {
   foreach my $spec (@$dirs) {
     my $dir = $self->localize_file_path($spec);
     next unless -e $dir;
-    do { $files{$_} = $_ if $self->contains_pod( $_ ) }
+    do { $files{$_} = File::Spec->abs2rel($_, $dir) if $self->contains_pod( $_ ) }
       for @{ $self->rscan_dir( $dir ) };
   }
   return \%files;
@@ -1345,17 +1337,13 @@ sub man1page_name {
 #    -spurkis
 sub man3page_name {
   my $self = shift;
-  my $file = File::Spec->canonpath( shift ); # clean up file path
-  my @dirs = File::Spec->splitdir( $file );
-
-  # more clean up - assume all man3 pods are under 'blib/lib' or 'blib/arch'
-  # to avoid the complexity found in Pod::Man::begin_pod()
-  shift @dirs while ($dirs[0] =~ /^(?:blib|lib|arch)$/i);
-
-  # remove known exts from the base name
-  $dirs[-1] =~ s/\.p(?:od|m|l)\z//i;
-
-  return join( $self->manpage_separator, @dirs );
+  my ($vol, $dirs, $file) = File::Spec->splitpath( shift );
+  my @dirs = File::Spec->splitdir( File::Spec->canonpath($dirs) );
+  
+  # Remove known exts from the base name
+  $file =~ s/\.p(?:od|m|l)\z//i;
+  
+  return join( $self->manpage_separator, @dirs, $file );
 }
 
 sub manpage_separator {
@@ -1373,6 +1361,7 @@ sub ACTION_diff {
   
   my $installmap = $self->install_map;
   delete $installmap->{read};
+  delete $installmap->{write};
 
   my $text_suffix = qr{\.(pm|pod)$};
 
@@ -1394,10 +1383,10 @@ sub ACTION_diff {
       next if $status == 0;  # Files are the same
       die "Can't compare $installed and $file: $!" if $status == -1;
       
-      if ($file !~ /$text_suffix/) {
-	print "Binary files $file and $installed differ\n";
-      } else {
+      if ($file =~ $text_suffix) {
 	$self->do_system('diff', @flags, $installed, $file);
+      } else {
+	print "Binary files $file and $installed differ\n";
       }
     }
   }
@@ -1544,16 +1533,20 @@ sub ACTION_distdir {
   }
   
   my $dist_files = $self->_read_manifest('MANIFEST');
+  delete $dist_files->{SIGNATURE};  # Don't copy, create a fresh one
   unless (keys %$dist_files) {
     warn "No files found in MANIFEST - try running 'manifest' action?\n";
     return;
   }
+  warn "*** Did you forget to add $self->{metafile} to the MANIFEST?\n" unless exists $dist_files->{$self->{metafile}};
   
   my $dist_dir = $self->dist_dir;
   $self->delete_filetree($dist_dir);
   $self->add_to_cleanup($dist_dir);
-  ExtUtils::Manifest::manicopy($dist_files, $dist_dir, 'cp');
-  warn "*** Did you forget to add $self->{metafile} to the MANIFEST?\n" unless exists $dist_files->{$self->{metafile}};
+  
+  foreach my $file (keys %$dist_files) {
+    $self->copy_if_modified(from => $file, to_dir => $dist_dir, verbose => 0);
+  }
   
   $self->_sign_dir($dist_dir) if $self->{properties}{sign};
 }
@@ -1633,11 +1626,7 @@ sub ACTION_distmeta {
     $node->{$_} = $p->{$_} if exists $p->{$_};
   }
   
-  $node->{provides} = $self->find_dist_packages
-    or do {
-      warn "Module::Info was not available, no 'provides' will be created in $self->{metafile}";
-      delete $node->{provides};
-    };
+  $node->{provides} = $self->find_dist_packages;
 
   $node->{generated_by} = "Module::Build version " . Module::Build->VERSION;
   
@@ -1800,6 +1789,34 @@ sub autosplit_file {
   AutoSplit::autosplit($file, $dir);
 }
 
+sub have_c_compiler {
+  my ($self) = @_;
+  my $p = $self->{properties}; 
+  return $p->{have_compiler} if defined $p->{have_compiler};
+  
+  print "Checking if compiler tools configured... " if $p->{verbose};
+  
+  my $c_file = $self->config_file('compilet.c');
+  {
+    my $fh = IO::File->new("> $c_file") or die "Can't create $c_file: $!";
+    print $fh "int boot_compilet() { return 1; }\n";
+  }
+  
+  my ($obj_file, $lib_file);
+  eval {
+    local $p->{module_name} = 'compilet';  # Fool compile_c() and link_c() about the library name
+    $obj_file = $self->compile_c($c_file);
+    (my $file_base = $obj_file) =~ s/\.[^.]+$//;
+    $file_base =~ tr/"//d;
+    $lib_file = $self->link_c($self->config_dir, $file_base);
+  };
+  unlink for grep defined, $c_file, $obj_file, $lib_file;
+  
+  my $result = $p->{have_compiler} = $@ ? 0 : 1;
+  print($result ? "ok.\n" : "failed.\n") if $p->{verbose};
+  return $result;
+}
+
 sub compile_c {
   my ($self, $file) = @_;
   my ($cf, $p) = ($self->{config}, $self->{properties}); # For convenience
@@ -1814,16 +1831,47 @@ sub compile_c {
 				   File::Spec->catdir($cf->{installarchlib}, 'CORE'));
   
   my @extra_compiler_flags = $self->split_like_shell($p->{extra_compiler_flags});
+  my @cccdlflags = $self->split_like_shell($cf->{cccdlflags});
   my @ccflags = $self->split_like_shell($cf->{ccflags});
   my @optimize = $self->split_like_shell($cf->{optimize});
+  my @flags = (@include_dirs, @cccdlflags, @extra_compiler_flags, '-c', @ccflags, @optimize);
+  
   my @cc = $self->split_like_shell($cf->{cc});
   
-  $self->do_system(@cc, @include_dirs, @extra_compiler_flags, '-c', @ccflags, @optimize, '-o', $obj_file, $file)
+  $self->do_system(@cc, @flags, '-o', $obj_file, $file)
     or die "error building $cf->{dlext} file from '$file'";
 
   return $obj_file;
 }
 
+# Most platforms don't need prelinking stuff done
+sub need_prelink_c { 0 }
+
+sub prelink_c {
+  my ($self, $to, $file_base) = @_;
+  my ($p, $args) = ($self->{properties}, $self->{args});
+
+  $file_base =~ tr/"//d; # remove any quotes
+  my $basename = File::Basename::basename($file_base);
+
+  print "ExtUtils::Mksymlists::Mksymlists('$file_base')\n";
+
+  require ExtUtils::Mksymlists;
+  ExtUtils::Mksymlists::Mksymlists( # dl. abbrev for dynamic library
+    NAME     => $args->{dl_name}      || $p->{module_name},
+    DLBASE   => $args->{dl_base}      || $basename,
+    DL_VARS  => $args->{dl_vars}      || [],
+    DL_FUNCS => $args->{dl_funcs}     || {},
+    FUNCLIST => $args->{dl_func_list} || [],
+    IMPORTS  => $args->{dl_imports}   || {},
+    FILE     => $file_base,
+  );
+
+  # *One* of these will be created by Mksymlists depending on $^O
+  local $_;
+  $self->add_to_cleanup("$file_base.$_") for qw(ext def opt);
+}
+
 sub link_c {
   my ($self, $to, $file_base) = @_;
   my ($cf, $p) = ($self->{config}, $self->{properties}); # For convenience
@@ -1833,6 +1881,8 @@ sub link_c {
   my $objects = $p->{objects} || [];
   
   unless ($self->up_to_date(["$file_base$cf->{obj_ext}", @$objects], $lib_file)) {
+    $self->prelink_c($to, $file_base) if $self->need_prelink_c;
+
     my @linker_flags = $self->split_like_shell($p->{extra_linker_flags});
     my @lddlflags = $self->split_like_shell($cf->{lddlflags});
     my @shrp = $self->split_like_shell($cf->{shrpenv});
@@ -1841,6 +1891,8 @@ sub link_c {
 		     "$file_base$cf->{obj_ext}", @$objects, @linker_flags)
       or die "error building $file_base$cf->{obj_ext} from '$file_base.$cf->{dlext}'";
   }
+  
+  return $lib_file;
 }
 
 sub compile_xs {
@@ -1963,7 +2015,10 @@ sub do_system {
 
 sub copy_if_modified {
   my $self = shift;
-  my %args = @_ > 3 ? @_ : ( from => shift, to_dir => shift, flatten => shift );
+  my %args = (@_ > 3
+	      ? ( verbose => 1, @_ )
+	      : ( from => shift, to_dir => shift, flatten => shift )
+	     );
   
   my $file = $args{from};
   unless (defined $file and length $file) {
@@ -1986,7 +2041,7 @@ sub copy_if_modified {
   # Create parent directories
   File::Path::mkpath(File::Basename::dirname($to_path), 0, 0777);
   
-  print "$file -> $to_path\n";
+  print "$file -> $to_path\n" if $args{verbose};
   File::Copy::copy($file, $to_path) or die "Can't copy('$file', '$to_path'): $!";
   return $to_path;
 }
@@ -2023,13 +2078,13 @@ Module::Build::Base - Default methods for Module::Build
 
 =head1 SYNOPSIS
 
-  please see the Module::Build documentation
+  Please see the Module::Build documentation.
 
 =head1 DESCRIPTION
 
 The C<Module::Build::Base> module defines the core functionality of
 C<Module::Build>.  Its methods may be overridden by any of the
-platform-independent modules in the C<Module::Build::Platform::>
+platform-dependent modules in the C<Module::Build::Platform::>
 namespace, but the intention here is to make this base module as
 platform-neutral as possible.  Nicely enough, Perl has several core
 tools available in the C<File::> namespace for doing this, so the task
@@ -2039,7 +2094,7 @@ Please see the C<Module::Build> documentation for more details.
 
 =head1 AUTHOR
 
-Ken Williams, ken@forum.swarthmore.edu
+Ken Williams, ken@mathforum.org
 
 =head1 SEE ALSO
 
